@@ -1,4 +1,4 @@
-/* $Id: gucharmap-charmap.c,v 1.46 2004/02/02 00:03:49 nlevitt Exp $ */
+/* $Id: gucharmap-charmap.c,v 1.52 2006/01/27 22:33:43 behdad Exp $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -58,7 +58,7 @@ charmap_finalize (GObject *object)
   gdk_cursor_unref (charmap->regular_cursor);
 }
 
-void
+static void
 gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
 {
   clazz->status_message = NULL;
@@ -144,19 +144,35 @@ insert_chocolate_detail_codepoints (GucharmapCharmap *charmap,
 #define is_hex_digit(c) (((c) >= '0' && (c) <= '9') \
                          || ((c) >= 'A' && (c) <= 'F'))
 
-/* returns a pointer to the start of [0-9A-F]{4}, or null if not found */
+/* - "XXXX-YYYY" used in annotation of U+003D 
+ * - Annotation of U+03C0 uses ".XXXX" as a fractional number,
+ *   so don't add "." to the list.
+ * Add here if you know more. */
+#define is_blank(c) (  ((c) == ' ')	\
+		    || ((c) == '-') )
+
+#define is_blank_or_hex_or(a,b) (  !((a) < len)	\
+				|| is_blank(str[a])	\
+				|| (is_hex_digit(str[a]) && (b)) )
+
+/* returns a pointer to the start of (?=[^ -])[0-9A-F]{4,5,6}[^0-9A-F],
+ * or null if not found */
 static const gchar *
 find_codepoint (const gchar *str)
 {
-  gint i;
+  guint i, len;
 
   /* what we are searching for is ascii; in this case, we don't have to
    * worry about multibyte characters at all */
-  for (i = 0;  i + 3 < strlen (str);  i++)
+  len = strlen (str);
+  for (i = 0;  i + 3 < len;  i++)
     {
-      if (is_hex_digit (str[i]) && is_hex_digit (str[i+1]) 
-          && is_hex_digit (str[i+2]) && is_hex_digit (str[i+3]))
-        return str + i;
+      if ( ( !(i > 0) || is_blank(str[i-1]) )
+	  && is_hex_digit (str[i+0]) && is_hex_digit (str[i+1]) 
+	  && is_hex_digit (str[i+2]) && is_hex_digit (str[i+3])
+	  && is_blank_or_hex_or(i+4,    is_blank_or_hex_or(i+5,
+		      (i+6 < len) || !is_hex_digit (str[i+6]))) )
+	return str + i;
     }
 
   return NULL;
@@ -237,7 +253,7 @@ conditionally_insert_canonical_decomposition (GucharmapCharmap *charmap,
 {
   gunichar *decomposition;
   gsize result_len;
-  gint i;
+  guint i;
 
   decomposition = gucharmap_unicode_canonical_decomposition (uc, &result_len);
 
@@ -272,10 +288,11 @@ set_details (GucharmapCharmap *charmap,
   gchar *temp;
   const gchar *csp;
   gchar buf[12];
-  guchar ubuf[7];
+  guchar utf8[7];
   gint n, i;
   const gchar **csarr;
   gunichar *ucs;
+  gunichar2 *utf16;
 
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (charmap->details));
   gtk_text_buffer_set_text (buffer, "", -1);
@@ -286,8 +303,8 @@ set_details (GucharmapCharmap *charmap,
 
   n = gucharmap_unichar_to_printable_utf8 (uc, buf);
   if (n == 0)
-    gtk_text_buffer_insert_with_tags_by_name (
-            buffer, &iter, _("[not a printable character]"), -1, NULL);
+    gtk_text_buffer_insert (
+            buffer, &iter, _("[not a printable character]"), -1);
   else
     gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, buf, n, 
                                               "gimongous", NULL);
@@ -310,37 +327,61 @@ set_details (GucharmapCharmap *charmap,
   /* canonical decomposition */
   conditionally_insert_canonical_decomposition (charmap, buffer, &iter, uc);
 
-  insert_heading (charmap, buffer, &iter, _("Various Useful Representations"));
+  /* representations */
+  if (g_unichar_type(uc) != G_UNICODE_BREAK_SURROGATE)
+    {
+      insert_heading (charmap, buffer, &iter, _("Various Useful Representations"));
 
-  n = g_unichar_to_utf8 (uc, ubuf);
+      n = g_unichar_to_utf8 (uc, (gchar *)utf8);
+      utf16 = g_ucs4_to_utf16 (&uc, 1, NULL, NULL, NULL);
 
-  /* UTF-8 */
-  gstemp = g_string_new (NULL);
-  for (i = 0;  i < n;  i++)
-    g_string_append_printf (gstemp, "0x%2.2X ", ubuf[i]);
-  g_string_erase (gstemp, gstemp->len - 1, -1);
-  insert_vanilla_detail (charmap, buffer, &iter, _("UTF-8:"), gstemp->str);
-  g_string_free (gstemp, TRUE);
+      /* UTF-8 */
+      gstemp = g_string_new (NULL);
+      for (i = 0;  i < n;  i++)
+	g_string_append_printf (gstemp, "0x%2.2X ", utf8[i]);
+      g_string_erase (gstemp, gstemp->len - 1, -1);
+      insert_vanilla_detail (charmap, buffer, &iter, _("UTF-8:"), gstemp->str);
+      g_string_free (gstemp, TRUE);
 
-  /* octal \012\234 UTF-8 */
-  gstemp = g_string_new (NULL);
-  for (i = 0;  i < n;  i++)
-    g_string_append_printf (gstemp, "\\%3.3o", ubuf[i]);
-  insert_vanilla_detail (charmap, buffer, &iter, 
-                         _("Octal escaped UTF-8:"), gstemp->str);
-  g_string_free (gstemp, TRUE);
+      /* UTF-16 */
+      gstemp = g_string_new (NULL);
+      g_string_append_printf (gstemp, "0x%4.4X", utf16[0]);
+      if (utf16[0] != '\0' && utf16[1] != '\0')
+	g_string_append_printf (gstemp, " 0x%4.4X", utf16[1]);
+      insert_vanilla_detail (charmap, buffer, &iter, _("UTF-16:"), gstemp->str);
+      g_string_free (gstemp, TRUE);
 
-  /* entity reference */
-  temp = g_strdup_printf ("&#%d;", uc);
-  insert_vanilla_detail (charmap, buffer, &iter, 
-                         _("Decimal entity reference:"), temp);
-  g_free (temp);
+      /* an empty line */
+      gtk_text_buffer_insert (buffer, &iter, "\n", -1);
 
+      /* C octal \012\234 */
+      gstemp = g_string_new (NULL);
+      for (i = 0;  i < n;  i++)
+	g_string_append_printf (gstemp, "\\%3.3o", utf8[i]);
+      insert_vanilla_detail (charmap, buffer, &iter, 
+			     _("C octal escaped UTF-8:"), gstemp->str);
+      g_string_free (gstemp, TRUE);
+
+      /* XML decimal entity */
+      if ((0x0001 <= uc && uc <= 0xD7FF) ||
+	  (0xE000 <= uc && uc <= 0xFFFD) ||
+	  (0x10000 <= uc && uc <= 0x10FFFF))
+	{
+	  temp = g_strdup_printf ("&#%d;", uc);
+	  insert_vanilla_detail (charmap, buffer, &iter, 
+				 _("XML decimal entity:"), temp);
+	  g_free (temp);
+	}
+
+      g_free(utf16);
+    }
+
+  /* annotations */
   if (_gucharmap_unicode_has_nameslist_entry (uc))
     {
       insert_heading (charmap, buffer, &iter, 
                       _("Annotations and Cross References"));
-    
+
       /* nameslist equals (alias names) */
       csarr = gucharmap_get_nameslist_equals (uc);
       if (csarr != NULL)
@@ -349,7 +390,7 @@ set_details (GucharmapCharmap *charmap,
                                    _("Alias names:"), csarr, FALSE);
           g_free (csarr);
         }
-    
+
       /* nameslist stars (notes) */
       csarr = gucharmap_get_nameslist_stars (uc);
       if (csarr != NULL)
@@ -358,7 +399,7 @@ set_details (GucharmapCharmap *charmap,
                                    _("Notes:"), csarr, TRUE);
           g_free (csarr);
         }
-    
+
       /* nameslist exes (see also) */
       ucs = gucharmap_get_nameslist_exes (uc);
       if (ucs != NULL)
@@ -367,7 +408,7 @@ set_details (GucharmapCharmap *charmap,
                                               _("See also:"), ucs);
           g_free (ucs);
         }
-    
+
       /* nameslist pounds (approximate equivalents) */
       csarr = gucharmap_get_nameslist_pounds (uc);
       if (csarr != NULL)
@@ -376,7 +417,7 @@ set_details (GucharmapCharmap *charmap,
                                    _("Approximate equivalents:"), csarr, TRUE);
           g_free (csarr);
         }
-    
+
       /* nameslist colons (equivalents) */
       csarr = gucharmap_get_nameslist_colons (uc);
       if (csarr != NULL)
@@ -391,6 +432,7 @@ set_details (GucharmapCharmap *charmap,
 
   /* this isn't so bad efficiency-wise */
   if (gucharmap_get_unicode_kDefinition (uc)
+      || gucharmap_get_unicode_kCantonese (uc)
       || gucharmap_get_unicode_kMandarin (uc)
       || gucharmap_get_unicode_kJapaneseOn (uc)
       || gucharmap_get_unicode_kJapaneseKun (uc)
@@ -409,6 +451,11 @@ set_details (GucharmapCharmap *charmap,
         insert_vanilla_detail (charmap, buffer, &iter,
                                _("Mandarin Pronunciation:"), csp);
     
+      csp = gucharmap_get_unicode_kCantonese (uc);
+      if (csp)
+        insert_vanilla_detail (charmap, buffer, &iter,
+                               _("Cantonese Pronunciation:"), csp);
+    
       csp = gucharmap_get_unicode_kJapaneseOn (uc);
       if (csp)
         insert_vanilla_detail (charmap, buffer, &iter,
@@ -520,9 +567,10 @@ follow_if_link (GucharmapCharmap *charmap,
 
       if (uc != (gunichar)(-1)) 
         {
-          g_signal_emit (charmap, gucharmap_charmap_signals[LINK_CLICKED], 
-                          0, charmap->chartable->active_cell, uc);
-          gucharmap_table_set_active_character (charmap->chartable, uc);
+          g_signal_emit (charmap, gucharmap_charmap_signals[LINK_CLICKED], 0, 
+                         gucharmap_table_get_active_character (charmap->chartable), 
+                         uc);
+          gucharmap_charmap_go_to_character (charmap, uc);
           break;
         }
     }
@@ -786,7 +834,7 @@ gucharmap_charmap_get_type (void)
         NULL,           /* class_data */
         sizeof (GucharmapCharmap),
         0,              /* n_preallocs */
-        (GInstanceInitFunc) gucharmap_charmap_init,
+        (GInstanceInitFunc) gucharmap_charmap_init
       };
 
       gucharmap_charmap_type = g_type_register_static (GTK_TYPE_HPANED, 
@@ -817,7 +865,7 @@ gucharmap_charmap_go_to_character (GucharmapCharmap *charmap,
   if (!status)
     g_warning ("gucharmap_chapters_go_to_character failed (%04X)\n", wc);
 
-  if (wc >= 0 && wc <= UNICHAR_MAX)
+  if (wc <= UNICHAR_MAX)
     gucharmap_table_set_active_character (charmap->chartable, wc);
 }
 
