@@ -1,4 +1,4 @@
-/* $Id: gucharmap-window.c,v 1.62 2005/11/28 21:42:07 nlevitt Exp $ */
+/* $Id: gucharmap-window.c,v 1.65 2006/01/27 22:33:43 behdad Exp $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -335,8 +335,8 @@ help_about (GtkAction       *action,
     };	  
   const gchar *translator_credits;
 
-  translator_credits = _("translator_credits");
-  if (strcmp (translator_credits, "translator_credits") == 0)
+  translator_credits = _("translator-credits");
+  if (strcmp (translator_credits, "translator-credits") == 0)
     translator_credits = NULL;
 
   gtk_show_about_dialog (GTK_WINDOW (guw),
@@ -356,6 +356,7 @@ prev_character (GtkAction       *action,
                 GucharmapWindow *guw)
 {
   gint index = guw->charmap->chartable->active_cell;
+  gint start = index;
   gunichar wc;
 
   do
@@ -367,7 +368,7 @@ prev_character (GtkAction       *action,
 
       wc = gucharmap_codepoint_list_get_char (guw->charmap->chartable->codepoint_list, index);
     }
-  while (!gucharmap_unichar_isdefined (wc) || !gucharmap_unichar_validate (wc));
+  while ((!gucharmap_unichar_isdefined (wc) || !gucharmap_unichar_validate (wc)) && index != start);
 
   gucharmap_table_set_active_character (guw->charmap->chartable, wc);
 }
@@ -377,6 +378,7 @@ next_character (GtkAction       *action,
                 GucharmapWindow *guw)
 {
   gint index = guw->charmap->chartable->active_cell;
+  gint start = index;
   gunichar wc;
 
   do
@@ -388,7 +390,7 @@ next_character (GtkAction       *action,
 
       wc = gucharmap_codepoint_list_get_char (guw->charmap->chartable->codepoint_list, index);
     }
-  while (!gucharmap_unichar_isdefined (wc) || !gucharmap_unichar_validate (wc));
+  while ((!gucharmap_unichar_isdefined (wc) || !gucharmap_unichar_validate (wc)) && index != start);
 
   gucharmap_table_set_active_character (guw->charmap->chartable, wc);
 }
@@ -575,6 +577,13 @@ make_menu (GucharmapWindow *guw)
   			       gtk_ui_manager_get_accel_group (priv->uimanager) );
   
   priv->action_group = gtk_action_group_new ("gucharmap_actions");
+  gtk_action_group_set_translation_domain  (priv->action_group,
+					    GETTEXT_PACKAGE);
+  gtk_action_group_set_translate_func (priv->action_group,
+				       gucharmap_gettext,
+				       NULL,
+				       NULL);
+
   gtk_action_group_add_actions (priv->action_group,
   				menu_entries,
 				G_N_ELEMENTS (menu_entries),
